package org.apache.bcel.classfile;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.io.DataInput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Arrays;
import org.apache.bcel.classfile.Attribute;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.classfile.ModuleExports;
import org.apache.bcel.classfile.ModuleOpens;
import org.apache.bcel.classfile.ModuleProvides;
import org.apache.bcel.classfile.ModuleRequires;
import org.apache.bcel.classfile.Utility;
import org.apache.bcel.classfile.Visitor;

public final class Module extends Attribute {
  private final int moduleNameIndex;
  
  private final int moduleFlags;
  
  private final int moduleVersionIndex;
  
  private ModuleRequires[] requiresTable;
  
  private ModuleExports[] exportsTable;
  
  private ModuleOpens[] opensTable;
  
  private final int usesCount;
  
  private final int[] usesIndex;
  
  private ModuleProvides[] providesTable;
  
  Module(int nameIndex, int length, DataInput input, ConstantPool constantPool) throws IOException {
    super((byte)22, nameIndex, length, constantPool);
    this.moduleNameIndex = input.readUnsignedShort();
    this.moduleFlags = input.readUnsignedShort();
    this.moduleVersionIndex = input.readUnsignedShort();
    int requiresCount = input.readUnsignedShort();
    this.requiresTable = new ModuleRequires[requiresCount];
    for (int i = 0; i < requiresCount; i++)
      this.requiresTable[i] = new ModuleRequires(input); 
    int exportsCount = input.readUnsignedShort();
    this.exportsTable = new ModuleExports[exportsCount];
    for (int j = 0; j < exportsCount; j++)
      this.exportsTable[j] = new ModuleExports(input); 
    int opensCount = input.readUnsignedShort();
    this.opensTable = new ModuleOpens[opensCount];
    int k;
    for (k = 0; k < opensCount; k++)
      this.opensTable[k] = new ModuleOpens(input); 
    this.usesCount = input.readUnsignedShort();
    this.usesIndex = new int[this.usesCount];
    for (k = 0; k < this.usesCount; k++)
      this.usesIndex[k] = input.readUnsignedShort(); 
    int providesCount = input.readUnsignedShort();
    this.providesTable = new ModuleProvides[providesCount];
    for (int m = 0; m < providesCount; m++)
      this.providesTable[m] = new ModuleProvides(input); 
  }
  
  public void accept(Visitor v) {
    v.visitModule(this);
  }
  
  public Attribute copy(ConstantPool constantPool) {
    org.apache.bcel.classfile.Module c = (org.apache.bcel.classfile.Module)clone();
    c.requiresTable = new ModuleRequires[this.requiresTable.length];
    Arrays.setAll(c.requiresTable, i -> this.requiresTable[i].copy());
    c.exportsTable = new ModuleExports[this.exportsTable.length];
    Arrays.setAll(c.exportsTable, i -> this.exportsTable[i].copy());
    c.opensTable = new ModuleOpens[this.opensTable.length];
    Arrays.setAll(c.opensTable, i -> this.opensTable[i].copy());
    c.providesTable = new ModuleProvides[this.providesTable.length];
    Arrays.setAll(c.providesTable, i -> this.providesTable[i].copy());
    c.setConstantPool(constantPool);
    return c;
  }
  
  public void dump(DataOutputStream file) throws IOException {
    super.dump(file);
    file.writeShort(this.moduleNameIndex);
    file.writeShort(this.moduleFlags);
    file.writeShort(this.moduleVersionIndex);
    file.writeShort(this.requiresTable.length);
    for (ModuleRequires entry : this.requiresTable)
      entry.dump(file); 
    file.writeShort(this.exportsTable.length);
    for (ModuleExports entry : this.exportsTable)
      entry.dump(file); 
    file.writeShort(this.opensTable.length);
    for (ModuleOpens entry : this.opensTable)
      entry.dump(file); 
    file.writeShort(this.usesIndex.length);
    for (int entry : this.usesIndex)
      file.writeShort(entry); 
    file.writeShort(this.providesTable.length);
    for (ModuleProvides entry : this.providesTable)
      entry.dump(file); 
  }
  
  public ModuleExports[] getExportsTable() {
    return this.exportsTable;
  }
  
  public ModuleOpens[] getOpensTable() {
    return this.opensTable;
  }
  
  public ModuleProvides[] getProvidesTable() {
    return this.providesTable;
  }
  
  public ModuleRequires[] getRequiresTable() {
    return this.requiresTable;
  }
  
  public String toString() {
    ConstantPool cp = getConstantPool();
    StringBuilder buf = new StringBuilder();
    buf.append("Module:\n");
    buf.append("  name:    ").append(Utility.pathToPackage(cp.getConstantString(this.moduleNameIndex, (byte)19))).append("\n");
    buf.append("  flags:   ").append(String.format("%04x", new Object[] { Integer.valueOf(this.moduleFlags) })).append("\n");
    String version = (this.moduleVersionIndex == 0) ? "0" : cp.getConstantString(this.moduleVersionIndex, (byte)1);
    buf.append("  version: ").append(version).append("\n");
    buf.append("  requires(").append(this.requiresTable.length).append("):\n");
    for (ModuleRequires module : this.requiresTable)
      buf.append("    ").append(module.toString(cp)).append("\n"); 
    buf.append("  exports(").append(this.exportsTable.length).append("):\n");
    for (ModuleExports module : this.exportsTable)
      buf.append("    ").append(module.toString(cp)).append("\n"); 
    buf.append("  opens(").append(this.opensTable.length).append("):\n");
    for (ModuleOpens module : this.opensTable)
      buf.append("    ").append(module.toString(cp)).append("\n"); 
    buf.append("  uses(").append(this.usesIndex.length).append("):\n");
    for (int index : this.usesIndex) {
      String className = cp.getConstantString(index, (byte)7);
      buf.append("    ").append(Utility.compactClassName(className, false)).append("\n");
    } 
    buf.append("  provides(").append(this.providesTable.length).append("):\n");
    for (ModuleProvides module : this.providesTable)
      buf.append("    ").append(module.toString(cp)).append("\n"); 
    return buf.substring(0, buf.length() - 1);
  }
}
